@@ -67,13 +67,13 @@
 	    format/1,
 	    garbage_collect/0,
 	    arithmetic_function/1,
-	    default_module/2,
 	    absolute_file_name/2,
 	    require/1,
 	    call_with_depth_limit/3,
 	    length/2,			% ?List, ?Length
 	    numbervars/3,		% +Term, +Start, -End
-	    nb_setval/2			% +Var, +Value
+	    nb_setval/2,		% +Var, +Value
+	    create_prolog_flag/3
 	  ]).
 
 		/********************************
@@ -390,7 +390,7 @@ prolog_load_context(directory, D) :-
 	source_location(F, _),
 	file_directory_name(F, D).
 prolog_load_context(dialect, D) :-
-	'$set_dialect'(D, D).
+	current_prolog_flag(emulated_dialect, D).
 prolog_load_context(term_position, '$stream_position'(0,L,0,0,0)) :-
 	source_location(_, L).
 
@@ -900,16 +900,6 @@ arithmetic_function(Module:Term) :-
 	functor(Head, Name, PredArity),
 	'$arithmetic_function'(Module:Head, 0).
 
-%%	default_module(+Me, -Super) is nondet.
-%
-%	Is true if `Super' is `Me' or a super (auto import) module of `Me'.
-
-default_module(Me, Me).
-default_module(Me, Super) :-
-	import_module(Me, S),
-	default_module(S, Super).
-
-
 		 /*******************************
 		 *	 LIST MANIPULATION	*
 		 *******************************/
@@ -966,3 +956,10 @@ numbervars(Term, From, To) :-
 nb_setval(Name, Value) :-
 	duplicate_term(Value, Copy),
 	nb_linkval(Name, Copy).
+
+%%	create_prolog_flag(Name, Value, Options)
+%
+%	`Forward' compatibility
+
+create_prolog_flag(Name, Value, _Options) :-
+	set_prolog_flag(Name, Value).
