@@ -30,19 +30,18 @@
 */
 
 :- module('$toplevel',
-	[ '$initialise'/0		% start Prolog (does not return)
-	, '$toplevel'/0			% Prolog top-level (re-entrant)
-	, '$abort'/0 			% restart after an abort
-	, '$break'/0 			% live in a break
-	, '$compile'/0 			% `-c' toplevel
-	, '$welcome'/0			% banner
-	, prolog/0 			% user toplevel predicate
-	, '$set_prompt'/1			% set the main prompt
-	, at_initialization/1		% goals to run at initialization
-	, (initialization)/1		% initialization goal (directive)
-	, '$thread_init'/0		% initialise thread
-	, (thread_initialization)/1	% thread initialization goal
-	]).
+	  [ '$initialise'/0,		% start Prolog (does not return)
+	    '$toplevel'/0,		% Prolog top-level (re-entrant)
+	    '$abort'/0,			% restart after an abort
+	    '$break'/0,			% live in a break
+	    '$compile'/0,		% `-c' toplevel
+	    '$welcome'/0,		% banner
+	    prolog/0, 			% user toplevel predicate
+	    '$set_prompt'/1,		% set the main prompt
+	    (initialization)/1,		% initialization goal (directive)
+	    '$thread_init'/0,		% initialise thread
+	    (thread_initialization)/1	% thread initialization goal
+	    ]).
 
 
 		/********************************
@@ -112,37 +111,43 @@
 		 *******************************/
 
 :- meta_predicate
-	at_initialization(0),
 	initialization(0).
-:- dynamic
-	'$at_initialization'/1.
 
-%%	at_initialization(:Goal)
+:- '$iso'((initialization)/1).
+
+%%	initialization(:Goal)
 %
-%	Runs Goal after loading a saved state.
-
-at_initialization(Goal) :-
-	assert('$at_initialization'(Goal)).
-
-'$run_at_initialization' :-
-	(   '$at_initialization'(Goal),
-	    (   catch(Goal, E,
-		      print_message(error, initialization_exception(Goal, E)))
-	    ->  fail
-	    ;   print_message(warning, goal_failed(at_initialization, Goal)),
-		fail
-	    )
-	;   true
-	),
-	'$thread_init'.
-
-%%	initialization(+Goal)
+%	Runs Goal after loading the file in which this directive
+%	appears as well as after restoring a saved state.
 %
-%	Runs `Goal' immediately and after loading a saved state.
+%	@see initialization/2
 
 initialization(Goal) :-
-	at_initialization(Goal),
-	call(Goal).
+	Goal = _:G,
+	prolog:initialize_now(G, Use), !,
+	print_message(warning, initialize_now(G, Use)),
+	initialization(Goal, now).
+initialization(Goal) :-
+	initialization(Goal, after_load).
+
+:- multifile
+	prolog:initialization_now/2,
+	prolog:message//1.
+
+prolog:initialize_now(load_foreign_library(_),
+		      'use :- use_foreign_library/1 instead').
+prolog:initialize_now(load_foreign_library(_,_),
+		      'use :- use_foreign_library/2 instead').
+
+prolog:message(initialize_now(Goal, Use)) -->
+	[ 'Initialization goal ~p will be executed'-[Goal],nl,
+	  'immediately for backward compatibility reasons', nl,
+	  '~w'-[Use]
+	].
+
+'$run_initialization' :-
+	'$run_initialization'(_),
+	'$thread_init'.
 
 
 		 /*******************************
@@ -338,7 +343,7 @@ initialise_prolog :-
 	set_prolog_flag(prompt_alternatives_on, determinism),
 	set_prolog_flag(toplevel_extra_white_line, true),
 	'$set_debugger_print_options'(print),
-	'$run_at_initialization',
+	'$run_initialization',
 	'$load_system_init_file',
 	'$load_gnu_emacs_interface',
 	'$option'(init_file, OsFile, OsFile),
@@ -396,7 +401,7 @@ initialise_prolog :-
 %	Toplevel called when invoked with -c option.
 
 '$compile' :-
-	'$run_at_initialization',
+	'$run_initialization',
 	'$load_system_init_file',
 	'$set_file_search_paths',
 	catch('$compile_wic', E, (print_message(error, E), halt(1))).
@@ -490,6 +495,32 @@ set_default_history :-
 :- initialization set_default_history.
 
 
+		 /*******************************
+		 *	  TOPLEVEL DEBUG	*
+		 *******************************/
+
+save_debug :-
+	(   tracing,
+	    notrace
+	->  Tracing = true
+	;   Tracing = false
+	),
+	current_prolog_flag(debug, Debugging),
+	set_prolog_flag(debug, false),
+	set_prolog_flag(query_debug_settings, debug(Debugging, Tracing)).
+
+restore_debug :-
+	current_prolog_flag(query_debug_settings, debug(Debugging, Tracing)),
+	set_prolog_flag(debug, Debugging),
+	(   Tracing == true
+	->  trace
+	;   true
+	).
+
+:- initialization
+	set_prolog_flag(query_debug_settings, debug(false, false)).
+
+
 		/********************************
 		*            PROMPTING		*
 		********************************/
@@ -515,9 +546,10 @@ set_default_history :-
 	->   '$substitute'("%l", ["[", BrekLev, "] "], P1, P2)
 	;    '$substitute'("%l", [], P1, P2)
 	),
-	(    tracing
+	current_prolog_flag(query_debug_settings, debug(Debugging, Tracing)),
+	(    Tracing == true
 	->   '$substitute'("%d", ["[trace] "], P2, P3)
-	;    current_prolog_flag(debug, true)
+	;    Debugging == true
 	->   '$substitute'("%d", ["[debug] "], P2, P3)
 	;    '$substitute'("%d", [], P2, P3)
 	),
@@ -556,34 +588,31 @@ subst_chars([H|T]) -->
 	print_message(query, query(eof)).
 '$execute'(Goal, Bindings) :-
 	'$module'(TypeIn, TypeIn),
-	expand_goal(Goal, Expanded),
+	setup_call_cleanup('$set_source_module'(M0, TypeIn),
+			   expand_goal(Goal, Expanded),
+			   '$set_source_module'(_, M0)),
 	'$dwim_correct_goal'(TypeIn:Expanded, Bindings, Corrected), !,
-	'$execute_goal'(Corrected, Bindings).
+	print_message(silent, toplevel_goal(Goal, Bindings)),
+	'$execute_goal2'(Corrected, Bindings).
 '$execute'(_, _) :-
 	notrace,
 	print_message(query, query(no)),
 	fail.
 
-'$execute_goal'(trace, []) :-
-	trace,
-	print_message(query, query(yes)), !,
-	fail.
-'$execute_goal'(Goal, Bindings) :-
-	print_message(silent, toplevel_goal(Goal, Bindings)),
-	'$execute_goal2'(Goal, Bindings).
-
 '$execute_goal2'(Goal, Bindings) :-
+	restore_debug,
 	Goal,
-	flush_output(user_output),
 	deterministic(Det),
+	(   save_debug
+	;   restore_debug, fail
+	),
+	flush_output(user_output),
 	call_expand_answer(Bindings, NewBindings),
-	(    write_bindings(NewBindings, Det)
-	->   !,
-	     notrace,
-	     fail
+	(    \+ \+ write_bindings(NewBindings, Det)
+	->   !, fail
 	).
 '$execute_goal2'(_, _) :-
-	notrace,
+	save_debug,
 	print_message(query, query(no)),
 	fail.
 
@@ -595,7 +624,7 @@ subst_chars([H|T]) -->
 %	=determinism= is considered more adequate and informative.
 
 write_bindings(Bindings, Det) :-
-	'$attributed'(Bindings), !,
+	\+ term_attvars(Bindings, []), !,
 	copy_term(Bindings, Bindings1, Residuals0),
 	'$module'(TypeIn, TypeIn),
 	omit_qualifiers(Residuals0, TypeIn, Residuals),
@@ -735,13 +764,11 @@ answer_respons(-1, show_again) :- !,
 answer_respons(Char, again) :-
 	print_message(query, no_action(Char)).
 
-print_predicate(0'w, [write], [ quoted(true),
-				priority(699)
+print_predicate(0'w, [write], [ quoted(true)
 			      ]).
 print_predicate(0'p, [print], [ quoted(true),
 				portray(true),
-				max_depth(10),
-				priority(699)
+				max_depth(10)
 			      ]).
 
 
