@@ -253,11 +253,6 @@ make_C_prototype(Out, Head) :-
 make_C_prototype(_, _).
 
 
-term_member(A, A).
-term_member(A, B) :-
-	arg(_, B, C),
-	term_member(A, C).
-
 %%	make_C_input_conversions(+Stream, +PrologHead)
 %
 %	Generate the input checking and conversion code.  Assumes
@@ -375,11 +370,6 @@ foreign_attributes(Head, Atts) :-
 	findall(A, foreign_attribute(Head, A), A0),
 	atomic_list_concat(A0, '|', Atts).
 
-insert_separator([], _, []).
-insert_separator([H], _, [H]).
-insert_separator([A|T], S, [A, S|NT]) :-
-	insert_separator(T, S, NT).
-
 foreign_attribute(Head, 'PL_FA_TRANSPARENT') :-
 	predicate_property(Head, transparent).
 
@@ -390,11 +380,11 @@ foreign_attribute(Head, 'PL_FA_TRANSPARENT') :-
 make_C_file_header(Out) :-
 	current_prolog_flag(version_data, swi(Major, Minor, Patch, _)),
 	get_time(Time),
-	convert_time(Time, Y, M, D, H, Min, _, _),
+	format_time(string(When), '%F %H:%M', Time),
 	format(Out, '/*  SWI-Prolog link wrapper~n', []),
 	format(Out, '    Generated by SWI-Prolog version ~w.~w.~w~n',
 	       [Major, Minor, Patch]),
-	format(Out, '    At ~w/~w/~w ~w:~w~n', [D, M, Y, H, Min]),
+	format(Out, '    At ~s', [When]),
 	(   source_location(File, Line)
 	->  format(Out, '    Source context ~w:~d~n', [File, Line])
 	;   true
@@ -472,7 +462,7 @@ build_shared_object(Object, Files, Libs) :-
 	file_name_extension(Object, Ext, SharedObject),
 	append(Files, Libs, Input),
 	atomic_list_concat(Input, ' ', InputAtom),
-	sformat(Command, 'plld -shared -o ~w ~w', [SharedObject, InputAtom]),
+	format(string(Command), 'plld -shared -o ~w ~w', [SharedObject, InputAtom]),
 	shell(Command).
 
 
