@@ -33,7 +33,8 @@
 	  [ init_win_menus/0
 	  ]).
 :- style_check(+dollar).
-:- op(100, fx, @).
+:- op(200, fy, @).
+:- op(990, xfx, :=).
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This library sets up the menu of PLWIN.EXE. It is called from the system
@@ -191,7 +192,7 @@ gather_arg(file(Mode, Title), File) :-
 		 File)).
 
 prolog_file_pattern(Pattern) :-
-	prolog_file_type(Ext, prolog),
+	user:prolog_file_type(Ext, prolog),
 	atom_concat('*.', Ext, Pattern).
 
 
