@@ -1,4 +1,4 @@
-/*  $Id: lock.c,v 1.8 2007/01/13 21:23:50 jan Exp $
+/*  $Id$
 
     Part of the SWI-Prolog Semweb package
 
@@ -68,7 +68,7 @@ emulation used in SWI-Prolog. We have   deleted the broadcast facilities
 of the CVs as this is not used in this code.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static int 
+static int
 win32_cond_init(win32_cond_t *cv)
 { cv->events[SIGNAL]    = CreateEvent(NULL, FALSE, FALSE, NULL);
   cv->waiters = 0;
@@ -86,7 +86,7 @@ win32_cond_destroy(win32_cond_t *cv)
 
 #define WAIT_INTR (-1)
 
-static int 
+static int
 win32_cond_wait(win32_cond_t *cv,
 		CRITICAL_SECTION *external_mutex)
 { int rc;
@@ -121,7 +121,7 @@ win32_cond_wait(win32_cond_t *cv,
 }
 
 
-static int 
+static int
 win32_cond_signal(win32_cond_t *cv)	/* must be holding associated mutex */
 { if ( cv->waiters > 0 )
     SetEvent(cv->events[SIGNAL]);
@@ -151,14 +151,15 @@ rdlock(rwlock *lock)
 
     return TRUE;
   }
-  
+
   lock->waiting_readers++;
 
   for(;;)
   { int rc = win32_cond_wait(&lock->rdcondvar, &lock->mutex);
 
     if ( rc == WAIT_INTR )
-    { LeaveCriticalSection(&lock->mutex);
+    { lock->waiting_readers--;
+      LeaveCriticalSection(&lock->mutex);
       return FALSE;
     } else if ( rc == 0 )
     { if ( lock->allow_readers == TRUE )
@@ -183,7 +184,7 @@ wrlock(rwlock *lock, int allow_readers)
   }
 
   EnterCriticalSection(&lock->mutex);
-  
+
   if ( lock->writer == -1 && lock->readers == 0 )
   { ok:
 
@@ -209,7 +210,8 @@ wrlock(rwlock *lock, int allow_readers)
   { int rc = win32_cond_wait(&lock->wrcondvar, &lock->mutex);
 
     if ( rc == WAIT_INTR )
-    { LeaveCriticalSection(&lock->mutex);
+    { lock->waiting_writers--;
+      LeaveCriticalSection(&lock->mutex);
       return FALSE;
     } else if ( rc == 0 )
     { if ( lock->writer == -1 && lock->readers == 0 )
@@ -219,14 +221,14 @@ wrlock(rwlock *lock, int allow_readers)
     } else
     { assert(0);			/* TBD: OS errors */
     }
-  }     
+  }
 }
 
 
 int
 lockout_readers(rwlock *lock)
 { EnterCriticalSection(&lock->mutex);
-  
+
   if ( lock->readers == 0 )
   { ok:
 
@@ -242,7 +244,8 @@ lockout_readers(rwlock *lock)
   { int rc = win32_cond_wait(&lock->upcondvar, &lock->mutex);
 
     if ( rc == WAIT_INTR )
-    { LeaveCriticalSection(&lock->mutex);
+    { lock->waiting_upgrade--;
+      LeaveCriticalSection(&lock->mutex);
       return FALSE;
     } else if ( rc == 0 )
     { if ( lock->readers == 0 )
@@ -291,7 +294,7 @@ unlock(rwlock *lock, int rd)
     waiting = (lock->waiting_upgrade ? UPGRADE :
 	       lock->waiting_writers ? WRITE :
 	       lock->waiting_readers ? READ : NONE);
-  
+
     switch(waiting)
     { case UPGRADE:
 	win32_cond_signal(&lock->upcondvar);
@@ -324,7 +327,7 @@ lock_misc(rwlock *lock)
 int
 unlock_misc(rwlock *lock)
 { LeaveCriticalSection(&lock->misc_mutex);
-  
+
   return TRUE;
 }
 
@@ -371,7 +374,7 @@ destroy_lock(rwlock *lock)
 
   free(lock->read_by_thread);
 
-  return TRUE;  
+  return TRUE;
 }
 
 #else /*__WINDOWS__*/
@@ -403,7 +406,7 @@ rdlock(rwlock *lock)
 
     return TRUE;
   }
-  
+
   lock->waiting_readers++;
 
   for(;;)
@@ -411,7 +414,10 @@ rdlock(rwlock *lock)
 
     if ( rc == EINTR )
     { if ( PL_handle_signals() < 0 )
+      { lock->waiting_readers--;
+	pthread_mutex_unlock(&lock->mutex);
 	return FALSE;
+      }
       continue;
     } else if ( rc == 0 )
     { if ( lock->allow_readers == TRUE )
@@ -444,7 +450,7 @@ wrlock(rwlock *lock, int allow_readers)
   }
 
   pthread_mutex_lock(&lock->mutex);
-  
+
   if ( lock->writer == -1 && lock->readers == 0 )
   { ok:
 
@@ -473,7 +479,10 @@ wrlock(rwlock *lock, int allow_readers)
 
     if ( rc == EINTR )
     { if ( PL_handle_signals() < 0 )
+      { lock->waiting_writers--;
+	pthread_mutex_unlock(&lock->mutex);
 	return FALSE;
+      }
       continue;
     } else if ( rc == 0 )
     { if ( lock->writer == -1 && lock->readers == 0 )
@@ -483,14 +492,14 @@ wrlock(rwlock *lock, int allow_readers)
     } else
     { assert(0);			/* TBD: OS errors */
     }
-  }     
+  }
 }
 
 
 int
 lockout_readers(rwlock *lock)
 { pthread_mutex_lock(&lock->mutex);
-  
+
   if ( lock->readers == 0 )
   { ok:
 
@@ -507,7 +516,10 @@ lockout_readers(rwlock *lock)
 
     if ( rc == EINTR )
     { if ( PL_handle_signals() < 0 )
+      { lock->waiting_upgrade--;
+	pthread_mutex_unlock(&lock->mutex);
 	return FALSE;
+      }
       continue;
     } else if ( rc == 0 )
     { if ( lock->readers == 0 )
@@ -557,7 +569,7 @@ unlock(rwlock *lock, int rd)		/* TRUE: read lock */
 	       lock->waiting_writers ? WRITE :
 	       lock->waiting_readers ? READ : NONE);
     pthread_mutex_unlock(&lock->mutex);
-  
+
     switch(waiting)
     { case UPGRADE:
 	pthread_cond_signal(&lock->upcondvar);
@@ -574,7 +586,7 @@ unlock(rwlock *lock, int rd)		/* TRUE: read lock */
   } else
   { pthread_mutex_unlock(&lock->mutex);
   }
-  
+
   return TRUE;
 }
 
@@ -708,7 +720,7 @@ reallow_readers(rwlock *lock)
 
 int
 destroy_lock(rwlock *lock)
-{ return TRUE;  
+{ return TRUE;
 }
 
 #endif /*_REENTRANT*/
