@@ -30,7 +30,7 @@
 #endif
 #include "lock.h"
 
-#define RDF_VERSION 20700		/* 2.7.0 */
+#define RDF_VERSION 20800		/* 2.8.0 */
 
 #define URL_subPropertyOf \
 	"http://www.w3.org/2000/01/rdf-schema#subPropertyOf"
@@ -68,7 +68,7 @@
 #define MAX_HASH_FACTOR 8		/* factor to trigger re-hash */
 #define MIN_HASH_FACTOR 4		/* factor after re-hash */
 
-#define NO_LINE	((unsigned long)-1L)
+#define NO_LINE	(0)
 
 typedef struct cell
 { void *	value;			/* represented resource */
@@ -100,7 +100,7 @@ typedef struct predicate
   list	            siblings;		/* reverse of subPropertyOf */
   int		    label;		/* Numeric label in cloud */
   struct predicate_cloud *cloud;	/* cloud I belong to */
-  unsigned long	    hash;		/* key used for hashing
+  unsigned int	    hash;		/* key used for hashing
   					   (=hash if ->cloud is up-to-date) */
 					/* properties */
   struct predicate *inverse_of;		/* my inverse predicate */
@@ -116,7 +116,7 @@ typedef struct predicate
 
 typedef struct predicate_cloud
 { predicate   **members;		/* member predicates */
-  unsigned long hash;			/* hash-code */
+  unsigned int  hash;			/* hash-code */
   int		size;			/* size of the cloud */
   int		deleted;		/* See erase_predicates() */
   bitmatrix    *reachable;		/* cloud reachability matrix */
@@ -128,12 +128,13 @@ typedef struct graph
 { struct graph    *next;		/* next in table */
   atom_t	    name;		/* name of the graph */
   atom_t	    source;		/* URL graph was loaded from */
+  double	    modified;		/* Modified time of source URL */
   int		    triple_count;	/* # triples associated to it */
 #ifdef WITH_MD5
   unsigned	    md5 : 1;		/* do/don't record MD5 */
   md5_byte_t 	    digest[16];		/* MD5 digest */
 #endif
-} graph;  
+} graph;
 
 
 typedef struct literal
