@@ -3,9 +3,9 @@
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2002, University of Amsterdam
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org/packages/xpce/
+    Copyright (C): 1985-2009, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -43,7 +43,7 @@
 		 *	    EXPANSION		*
 		 *******************************/
 
-user:term_expansion((:- require(_)), []).
+system:term_expansion((:- require(_)), []).
 
 
 		 /*******************************
@@ -102,7 +102,7 @@ pce_home(_) :-
 	halt(1).
 
 '$load_pce' :-
-	'$c_current_predicate'('$pce_init', user:'$pce_init'(_)), !,
+	current_predicate(user:'$pce_init'/1), !,
 	init_pce.
 '$load_pce' :-
 	current_prolog_flag(open_shared_object, true),
@@ -114,10 +114,13 @@ pce_home(_) :-
 	),
 	init_pce.
 
+init_pce :-
+	current_prolog_flag(xpce, true),
+	current_predicate(pce_principal:object/1), !.
 init_pce :-
 	(   pce_home(PceHome),
 	    pce_principal:'$pce_init'(PceHome)
-	->  set_prolog_flag(xpce, true)
+	->  create_prolog_flag(xpce, true, [])
 	;   print_message(error,
 			  format('Failed to initialise XPCE', [])),
 	    abort
@@ -148,5 +151,5 @@ prolog:message(pce(no_threads)) -->
 
 user:message_hook('$aborted', _Kind, _Lines) :-
 	current_prolog_flag(xpce, true),
-	send(@display, reset),
+	send(@(display), reset),
 	fail.
