@@ -94,15 +94,18 @@ reload_styles(M) :->
 	;   true
 	).
 
-colourise_term(M, Term:prolog, TermPos:prolog) :->
+colourise_term(M, Term:prolog, TermPos:prolog, Comments:prolog) :->
 	"Colourise the given term"::
 	send(M, setup_styles),
 	get(M, text_buffer, TB),
 	arg(1, TermPos, From),
 	arg(2, TermPos, To),
 	send(M, remove_syntax_fragments, From, To),
-	send(M, colourise_comments, From, To),
-	colourise_term(Term, TB, TermPos).
+	(   Comments == (-)
+	->  send(M, colourise_comments, From, To)
+	;   true
+	),
+	colourise_term(Term, TB, TermPos, Comments).
 
 colourise_buffer(M) :->
 	"Do cross-referencing and colourising of the whole buffer"::
@@ -113,17 +116,24 @@ colourise_buffer(M) :->
 	send_super(M, colourise_buffer),
 	send(M, setup_styles),
 	send(M, xref_buffer),
+	send(M, slot, warnings, 0),
+	send(M, slot, errors, 0),
 	send(M, report, progress, 'Colourising buffer ...'),
 	colourise_buffer(M),
-	send(M, colourise_comments),
+	get(M, errors, Errors),
 	statistics(runtime, [_,UsedMilliSeconds]),
 	Used is UsedMilliSeconds/1000,
 	get(Class, no_created, @on, NewCreated),
 	Created is NewCreated - OldCreated,
-	send(M, report, done,
-	     'done, %.2f seconds, %d fragments', Used, Created).
+	(   Errors =:= 0
+	->  send(M, report, done,
+		 'done, %.2f seconds, %d fragments', Used, Created)
+	;   send(M, report, status,
+		 'File contains %d errors', Errors)
+	).
 
 colourise_comments(M, From:[int], To:[int]) :->
+	debug(emacs, 'Colourising comments in ~p..~p', [From, To]),
 	get(M, text_buffer, TB),
 	send(TB, for_all_comments,
 	     message(@prolog, colour_item, comment, TB, @arg1, @arg2),
@@ -136,6 +146,11 @@ colourise_or_recenter(M) :->
 	;   send(M, colourise_buffer)
 	).
 
+show_syntax_error(M, Where:int, Message:name) :->
+	"Show syntax error position"::
+	get(M, text_buffer, TB),
+	show_syntax_error(TB, Where:Message).
+
 xref_buffer(M, Always:[bool]) :->
 	"Run the cross-referencer on buffer"::
 	get(M, text_buffer, TB),
@@ -173,15 +188,17 @@ colourise_buffer(Fd, M) :-
 	TB = @Src,
 	repeat,
 	    '$set_source_module'(SM, SM),
+	    character_count(Fd, Start),
 	    catch(read_term(Fd, Term,
 			    [ subterm_positions(TermPos),
 			      singletons(Singletons),
-			      module(SM)
+			      module(SM),
+			      comments(Comments)
 			    ]),
 		  E,
-		  syntax_error(E)),
+		  syntax_error(M, Fd, Start, E)),
 	    fix_operators(Term, Src),
-	    (	colourise_term(Term, TB, TermPos),
+	    (	colourise_term(Term, TB, TermPos, Comments),
 		send(M, mark_singletons, Term, Singletons, TermPos)
 	    ->	true
 	    ;	arg(1, TermPos, From),
@@ -216,11 +233,28 @@ emacs_push_op(P, T, N0) :-
 	push_op(P, T, N),
 	debug(emacs, ':- ~w.', [op(P,T,N)]).
 
-%	syntax_error(+Error)
+%%	syntax_error(+Mode, +Stream, +Start, +Error)
 %
-%	Print syntax errors if the debugging topic emacs is active.
+%	Deal with syntax errors while colouring the whole buffer.
+%	Performs the following tasks:
+%
+%	    * Colourise comments using shallow syntax
+%	    * Indicate the error position
+%	    * If debug =emacs= is active, print the message
 
-syntax_error(E) :-
+syntax_error(M, Stream, Start, E) :-
+	character_count(Stream, End),
+	send(M, colourise_comments, Start, End),
+	(   \+ get(M, show_syntax_errors, never),
+	    E = error(syntax_error(_), stream(_S, _Line, _LinePos, CharNo))
+	->  message_to_string(E, Msg),
+	    get(M, text_buffer, TB),
+	    show_syntax_error(TB, CharNo:Msg)
+	;   true
+	),
+	get(M, errors, E0),
+	E1 is E0 + 1,
+	send(M, slot, errors, E1),
 	(   debugging(emacs)
 	->  print_message(error, E)
 	;   true
@@ -271,7 +305,7 @@ process_use_module(File, Src) :-
 	;   true
 	).
 
-%	colourise(+TB, +Stream)
+%%	colourise(+TB, +Stream)
 %
 %	Read next term from the text_buffer and  colourise the syntax
 
@@ -281,16 +315,31 @@ colourise(TB, Fd) :-
 						Term,
 						Error,
 						_Singletons,
-						TermPos),
+						TermPos, Comments),
 	(   Error == none
-	->  colourise_term(Term, TB, TermPos)
+	->  colourise_term(Term, TB, TermPos, Comments)
 	;   show_syntax_error(TB, Error)
 	).
 
-show_syntax_error(TB, Pos:_Message) :-
-	get(TB, scan, Pos, line, 0, start, BOL),
-	get(TB, scan, Pos, line, 0, end, EOL),
-	colour_item(syntax_error, TB, BOL-EOL).
+show_syntax_error(TB, Pos:Message) :-
+	End is Pos + 1,
+	colour_item(syntax_error(Message), TB, Pos-End).
+
+colourise_term(Term, TB, TermPos, Comments) :-
+	colourise_comments(Comments, TB),
+	colourise_term(Term, TB, TermPos).
+
+colourise_comments(-, _).
+colourise_comments([], _).
+colourise_comments([H|T], TB) :-
+	colourise_comment(H, TB),
+	colourise_comments(T, TB).
+
+colourise_comment(Pos-Comment, TB) :-
+	stream_position_data(char_count, Pos, Start),
+	string_length(Comment, Len),
+	End is Start + Len + 1,
+	colour_item(comment, TB, Start-End).
 
 colourise_term(Term, TB, Pos) :-
 	term_colours(Term, FuncSpec-ArgSpecs), !,
@@ -825,7 +874,7 @@ pi_to_term(Name//Arity0, Term) :-
 	Arity is Arity0 + 2,
 	functor(Term, Name, Arity).
 
-%	colour_item(+Class, +TB, +Pos)
+%%	colour_item(+Class, +TB, +Pos)
 %
 %	colourise region if a style is defined for this class.
 
@@ -1095,7 +1144,7 @@ def_style(hook,			style(colour := blue,
 
 def_style(error,		style(background := orange)).
 def_style(type_error(_),	style(background := orange)).
-def_style(syntax_error,	  	style(background := red)).
+def_style(syntax_error(_),	style(background := orange)).
 
 :- dynamic
 	style_name/2.
@@ -1830,6 +1879,8 @@ identify_fragment(directory(Path), _, Summary) :-
 	new(Summary, string('Directory %s', Path)).
 identify_fragment(type_error(Type), _, Summary) :-
 	new(Summary, string('Type error: argument must be a %s', Type)).
+identify_fragment(syntax_error(Message), _, Summary) :-
+	new(Summary, string('%s', Message)).
 identify_fragment(module(Module), _, Summary) :-
 	module_property(Module, file(Path)),
 	new(Summary, string('Module %s loaded from %s', Module, Path)).
