@@ -214,6 +214,18 @@ PL_error(const char *pred, int arity, const char *msg, int id, ...)
 		      PL_TERM, pred);
       break;
     }
+    case ERR_MODIFY_THREAD_LOCAL_PROC:
+    { Procedure proc = va_arg(args, Procedure);
+      term_t pred = PL_new_term_ref();
+
+      unify_definition(pred, proc->definition, 0, GP_NAMEARITY|GP_HIDESYSTEM);
+      PL_unify_term(formal,
+		    PL_FUNCTOR, FUNCTOR_permission_error3,
+		      PL_ATOM, ATOM_modify,
+		      PL_ATOM, ATOM_thread_local_procedure,
+		      PL_TERM, pred);
+      break;
+    }
     case ERR_UNDEFINED_PROC:
     { Definition def = va_arg(args, Definition);
       Definition clr = va_arg(args, Definition);
@@ -700,6 +712,28 @@ PL_get_intptr_ex(term_t t, intptr_t *i)
 }
 
 
+int
+PL_get_size_ex(term_t t, size_t *i)
+{ int64_t val;
+
+  if ( !PL_get_int64_ex(t, &val) )
+    fail;
+  if ( val < 0 )
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN,
+		    ATOM_not_less_than_zero, t);
+#if SIZEOF_VOIDP < 8
+#if SIZEOF_LONG == SIZEOF_VOIDP
+  if ( val > (int64_t)ULONG_MAX )
+    return PL_error(NULL, 0, NULL, ERR_REPRESENTATION, ATOM_size_t);
+#endif
+#endif
+
+  *i = val;
+
+  return TRUE;
+}
+
+
 int
 PL_get_bool_ex(term_t t, int *i)
 { if ( PL_get_bool(t, i) )
