@@ -154,6 +154,7 @@ COMMON(void)		markAtomsFindall(PL_local_data_t *ld);
 
 /* pl-comp.c */
 COMMON(void) 		initWamTable(void);
+COMMON(void)		freeVarDefs(PL_local_data_t *ld);
 COMMON(void) 		get_head_and_body_clause(term_t clause,
 					 term_t head, term_t body,
 					 Module *m ARG_LD);
@@ -239,6 +240,7 @@ COMMON(int) 		PL_get_integer_ex(term_t t, int *i);
 COMMON(int) 		PL_get_long_ex(term_t t, long *i);
 COMMON(int) 		PL_get_int64_ex(term_t t, int64_t *i);
 COMMON(int) 		PL_get_intptr_ex(term_t t, intptr_t *i);
+COMMON(int) 		PL_get_size_ex(term_t t, size_t *i);
 COMMON(int) 		PL_get_bool_ex(term_t t, int *i);
 COMMON(int) 		PL_get_float_ex(term_t t, double *f);
 COMMON(int) 		PL_get_char_ex(term_t t, int *p, int eof);
@@ -318,9 +320,9 @@ COMMON(int) 		PL_unify__LD(term_t t1, term_t t2 ARG_LD);
 COMMON(int) 		PL_unify_integer__LD(term_t t1, intptr_t i ARG_LD);
 COMMON(int)		PL_unify_int64_ex__LD(term_t t1, int64_t ARG_LD);
 COMMON(int) 		PL_get_atom__LD(term_t t1, atom_t *a ARG_LD);
-COMMON(void) 		PL_put_atom__LD(term_t t1, atom_t a ARG_LD);
+COMMON(int) 		PL_put_atom__LD(term_t t1, atom_t a ARG_LD);
 COMMON(void) 		PL_put_integer__LD(term_t t1, long i ARG_LD);
-COMMON(void) 		PL_put_intptr__LD(term_t t1, intptr_t i ARG_LD);
+COMMON(int) 		PL_put_intptr__LD(term_t t1, intptr_t i ARG_LD);
 COMMON(int) 		PL_is_atomic__LD(term_t t ARG_LD);
 COMMON(int) 		PL_is_functor__LD(term_t t, functor_t f ARG_LD);
 COMMON(int) 		PL_is_variable__LD(term_t t ARG_LD);
@@ -336,7 +338,7 @@ COMMON(int) 		PL_unify_pointer__LD(term_t t, void *ptr ARG_LD);
 COMMON(int) 		PL_get_list__LD(term_t l, term_t h, term_t t ARG_LD);
 COMMON(int) 		PL_is_atom__LD(term_t t ARG_LD);
 COMMON(int) 		PL_unify_list__LD(term_t l, term_t h, term_t t ARG_LD);
-COMMON(void) 		PL_cons_list__LD(term_t l, term_t head, term_t tail
+COMMON(int) 		PL_cons_list__LD(term_t l, term_t head, term_t tail
 					 ARG_LD);
 COMMON(int)		PL_is_inf(term_t t);
 COMMON(int)		isUCSAtom(Atom a);
@@ -378,7 +380,8 @@ COMMON(word) 		pl_garbage_collect(term_t d);
 COMMON(Word) 		findGRef(int n);
 COMMON(size_t)		nextStackSizeAbove(size_t n);
 COMMON(int) 		growStacks(LocalFrame fr, Choice ch, Code PC,
-			   intptr_t lminfree, intptr_t gminfree, intptr_t tminfree);
+				   size_t lminfree, size_t gminfree,
+				   size_t tminfree);
 COMMON(void) 		clearUninitialisedVarsFrame(LocalFrame, Code);
 COMMON(word) 		check_foreign(void);	/* O_SECURE stuff */
 COMMON(void) 		markAtomsOnStacks(PL_local_data_t *ld);
@@ -408,9 +411,12 @@ COMMON(Module) 		lookupModule(atom_t name);
 COMMON(Module) 		isCurrentModule(atom_t name);
 COMMON(void) 		initModules(void);
 COMMON(int) 		isSuperModule(Module s, Module m);
+COMMON(int)		addSuperModule(Module m, Module s, int where);
+COMMON(int)		getUnknownModule(Module m);
 COMMON(Word) 		stripModule(Word term, Module *module ARG_LD);
 COMMON(bool) 		isPublicModule(Module module, Procedure proc);
-COMMON(int) 		declareModule(atom_t name, SourceFile sf, int line,
+COMMON(int) 		declareModule(atom_t name, atom_t super,
+				      SourceFile sf, int line,
 				      int rdef);
 COMMON(word) 		pl_current_module(term_t module, term_t file, control_t h);
 COMMON(word) 		pl_module(term_t old, term_t new);
@@ -426,14 +432,7 @@ COMMON(word) 		pl_set_prolog_hook(term_t module, term_t old, term_t new);
 COMMON(int) 		currentOperator(Module m, atom_t name, int kind,
 				int *type, int *priority);
 COMMON(int) 		priorityOperator(Module m, atom_t atom);
-COMMON(word) 		pl_current_op(term_t prec, term_t type, term_t name,
-			      control_t h);
-COMMON(word) 		pl_local_op(term_t prec, term_t type, term_t name,
-			    control_t h);
-COMMON(word) 		pl_op(term_t priority, term_t type, term_t name);
 COMMON(void) 		initOperators(void);
-COMMON(word) 		pl_builtin_op(term_t prec, term_t type, term_t name,
-			      control_t h);
 
 /* pl-os.c */
 COMMON(bool) 		initOs(void);
@@ -510,7 +509,7 @@ COMMON(word) 		pl_break(void);
 COMMON(word) 		pl_break1(term_t goal);
 COMMON(word) 		pl_notrace1(term_t goal);
 COMMON(int) 		callProlog(Module module, term_t goal, int flags, term_t *ex);
-COMMON(word) 		pl_abort(abort_type type);
+COMMON(int) 		abortProlog(abort_type type);
 COMMON(bool) 		prologToplevel(atom_t toplevel);
 COMMON(word) 		pl_metacut(void);
 COMMON(int)  		trap_gdb(void);
@@ -551,7 +550,7 @@ COMMON(word) 		pl_get_clause_attribute(term_t ref, term_t att, term_t value);
 COMMON(word) 		pl_get_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(word) 		pl_set_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(void) 		reindexDefinition(Definition def);
-COMMON(void) 		redefineProcedure(Procedure proc, SourceFile sf,
+COMMON(int) 		redefineProcedure(Procedure proc, SourceFile sf,
 					  unsigned int suppress);
 COMMON(void) 		startConsult(SourceFile f);
 COMMON(void) 		indexDefinition(Definition def, long pattern);
@@ -578,7 +577,8 @@ COMMON(int) 		setDynamicProcedure(Procedure proc, bool isdyn);
 /* pl-prof.c */
 COMMON(void) 		stopItimer(void);
 COMMON(bool) 		resetProfiler(void);
-COMMON(struct)  call_node* profCall(Definition def ARG_LD);
+COMMON(struct call_node*) profCall(Definition def ARG_LD);
+COMMON(void) 		profResumeParent(struct call_node *node ARG_LD);
 COMMON(void) 		profExit(struct call_node *node ARG_LD);
 COMMON(void) 		profRedo(struct call_node *node ARG_LD);
 COMMON(void)		profSetHandle(struct call_node *node, void *handle);
@@ -622,7 +622,7 @@ COMMON(void) 		install_rl(void);
 
 /* pl-setup.c */
 COMMON(void) 		setupProlog(void);
-COMMON(void)		endCritical__LD(ARG1_LD);
+COMMON(int)		endCritical__LD(ARG1_LD);
 COMMON(handler_t) 	set_sighandler(int sig, handler_t func);
 COMMON(void) 		blockSignals(sigset_t *mask);
 COMMON(void) 		allSignalMask(sigset_t *set);
@@ -633,20 +633,22 @@ COMMON(void) 		resetSignals(void);
 COMMON(void) 		cleanupSignals(void);
 COMMON(int)		handleSignals(Code PC);
 
-COMMON(int) 		initPrologStacks(intptr_t local,
-				 intptr_t global,
-				 intptr_t trail,
-				 intptr_t argument);
+COMMON(int) 		initPrologStacks(size_t local,
+					 size_t global,
+					 size_t trail,
+					 size_t argument);
 COMMON(void) 		initPrologLocalData(void);
 COMMON(void) 		deallocateStacks(void);
 COMMON(bool) 		restoreStack(Stack s);
-COMMON(void) 		trimStacks(ARG1_LD);
+COMMON(void) 		trimStacks(int resize ARG_LD);
 COMMON(void) 		resetStacks(void);
 COMMON(void) 		emptyStacks(void);
 COMMON(void) 		freeStacks(ARG1_LD);
 COMMON(void) 		freeLocalData(PL_local_data_t *ld);
-COMMON(word) 		pl_trim_stacks(void);
 COMMON(void) 		ensure_room_stack(Stack s, size_t n);
+#ifdef O_SHIFT_STACKS
+COMMON(void)		trim_stack(Stack s);
+#endif
 
 /* pl-sys.c */
 COMMON(word) 		pl_shell(term_t command, term_t status);
@@ -676,8 +678,6 @@ COMMON(word) 		pl_leash(term_t old, term_t new);
 COMMON(word) 		pl_visible(term_t old, term_t new);
 COMMON(word) 		pl_debuglevel(term_t old, term_t new);
 COMMON(word) 		pl_prolog_current_frame(term_t fr);
-COMMON(word) 		pl_prolog_frame_attribute(term_t fr, term_t key, term_t val);
-COMMON(foreign_t) 	pl_prolog_choice_attribute(term_t fr, term_t key, term_t val);
 COMMON(void) 		callEventHook(int ev, ...);
 COMMON(void)		PL_put_frame(term_t t, LocalFrame fr);
 
