@@ -797,7 +797,9 @@ assertProcedure(Procedure proc, Clause clause, int where ARG_LD)
 
   def->number_of_clauses++;
 #ifdef O_LOGICAL_UPDATE
+  PL_LOCK(L_MISC);
   clause->generation.created = ++GD->generation;
+  PL_UNLOCK(L_MISC);
   clause->generation.erased  = ~0L;	/* infinite */
 #endif
 
@@ -867,7 +869,8 @@ abolishProcedure(Procedure proc, Module module)
     resetProcedure(proc, TRUE);
   } else if ( true(def, P_THREAD_LOCAL) )
   { UNLOCKDEF(def);
-    endCritical;
+    if ( !endCritical )
+      return FALSE;
     return PL_error(NULL, 0, NULL, ERR_PERMISSION_PROC,
 		    ATOM_modify, ATOM_thread_local_procedure, proc);
   } else				/* normal Prolog procedure */
@@ -877,14 +880,11 @@ abolishProcedure(Procedure proc, Module module)
     { if ( def->references == 0 )
       { resetProcedure(proc, FALSE);
 	gcClausesDefinitionAndUnlock(def);
-	endCritical;
-	succeed;
+	return endCritical;
       } else				/* dynamic --> static */
       { UNLOCKDYNDEF(def);		/* release private lock */
 	setDynamicProcedure(proc, FALSE);
-	endCritical;
-
-	succeed;
+	return endCritical;
       }
     } else if ( true(def, NEEDSCLAUSEGC) )
     { registerDirtyDefinition(def);
@@ -893,9 +893,7 @@ abolishProcedure(Procedure proc, Module module)
     resetProcedure(proc, FALSE);
   }
   UNLOCKDEF(def);
-  endCritical;
-
-  succeed;
+  return endCritical;
 }
 
 
@@ -952,7 +950,7 @@ clause-chain. The clause itself is not  deleted,   this  task is left to
 retractClauseDefinition().
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
+static int
 unlinkClause(Definition def, Clause clause ARG_LD)
 { ClauseRef prev = NULL;
   ClauseRef c;
@@ -982,7 +980,7 @@ unlinkClause(Definition def, Clause clause ARG_LD)
     }
   }
 
-  endCritical;
+  return endCritical;
 }
 
 
@@ -993,7 +991,9 @@ the definition is always referenced.
 
 bool
 retractClauseDefinition(Definition def, Clause clause ARG_LD)
-{ LOCKDYNDEF(def);
+{ int rc;
+
+  LOCKDYNDEF(def);
   assert(true(def, DYNAMIC));
   if ( true(clause, ERASED) )
   { UNLOCKDYNDEF(def);
@@ -1016,14 +1016,16 @@ retractClauseDefinition(Definition def, Clause clause ARG_LD)
     { set(def, NEEDSCLAUSEGC);
     }
 #ifdef O_LOGICAL_UPDATE
+    PL_LOCK(L_MISC);
     clause->generation.erased = ++GD->generation;
+    PL_UNLOCK(L_MISC);
 #endif
     UNLOCKDYNDEF(def);
 
     succeed;
   }
 
-  unlinkClause(def, clause PASS_LD);
+  rc = unlinkClause(def, clause PASS_LD);
   UNLOCKDYNDEF(def);
 
 					/* as we do a call-back, we cannot */
@@ -1036,7 +1038,7 @@ retractClauseDefinition(Definition def, Clause clause ARG_LD)
 
   freeClause(clause PASS_LD);
 
-  succeed;
+  return rc;
 }
 
 
@@ -1677,7 +1679,7 @@ trapUndefined_unlocked(LocalFrame *frp, Code PC, Definition def ARG_LD)
     return newdef;
 					/* Pred/Module does not want to trap */
   if ( true(def, PROC_DEFINED) ||
-       false(module, UNKNOWN_WARNING|UNKNOWN_ERROR) )
+       getUnknownModule(module) == UNKNOWN_FAIL )
     return def;
 
   DEBUG(5, Sdprintf("trapUndefined(%s)\n", predicateName(def)));
@@ -1712,7 +1714,7 @@ trapUndefined_unlocked(LocalFrame *frp, Code PC, Definition def ARG_LD)
 error:
   if ( GD->bootsession )
   { sysError("Undefined predicate: %s", predicateName(def));
-  } else if ( true(module, UNKNOWN_ERROR) )
+  } else
   { createUndefSupervisor(def);
   }
 
@@ -1858,11 +1860,11 @@ PRED_IMPL("retract", 1, retract, PL_FA_TRANSPARENT|PL_FA_NONDETERMINISTIC)
       { retractClauseDefinition(getProcDefinition(proc), cref->clause PASS_LD);
 	if ( !next )
 	{ leaveDefinition(def);
-	  endCritical;
-	  succeed;
+	  return endCritical;
 	}
 
-	endCritical;
+	if ( !endCritical )
+	  fail;
 	ForeignRedoPtr(next);
       }
 
@@ -1916,9 +1918,9 @@ pl_retractall(term_t head)
   fid = PL_open_foreign_frame();
 
   if ( !(cref = firstClause(argv, fr, def, &next PASS_LD)) )
-  { endCritical;
+  { int rc = endCritical;
     leaveDefinition(def);
-    succeed;
+    return rc;
   }
 
   while( cref )
@@ -1929,17 +1931,14 @@ pl_retractall(term_t head)
 
     if ( !next )
     { leaveDefinition(def);
-      endCritical;
-      succeed;
+      return endCritical;
     }
 
 
     cref = findClause(next, argv, fr, def, &next PASS_LD);
   }
-  endCritical;
   leaveDefinition(def);
-
-  succeed;
+  return endCritical;
 }
 
 		/********************************
@@ -2630,7 +2629,7 @@ addProcedureSourceFile(SourceFile sf, Procedure proc)
 }
 
 
-void
+int
 redefineProcedure(Procedure proc, SourceFile sf, unsigned int suppress)
 { GET_LD
   Definition def = proc->definition;
@@ -2642,10 +2641,12 @@ redefineProcedure(Procedure proc, SourceFile sf, unsigned int suppress)
 		   PL_CHARS, "foreign",
 		   _PL_PREDICATE_INDICATOR, proc);
   }
-  assert(false(def, P_THREAD_LOCAL));	/* what to do? */
 
   if ( false(def, MULTIFILE) )
-  { ClauseRef first = hasClausesDefinition(def);
+  { ClauseRef first;
+
+    def = getProcDefinition__LD(def PASS_LD);
+    first = hasClausesDefinition(def);
 
     if ( first && first->clause->source_no == sf->index )
     { if ( ((debugstatus.styleCheck & ~suppress) & DISCONTIGUOUS_STYLE) &&
@@ -2654,7 +2655,10 @@ redefineProcedure(Procedure proc, SourceFile sf, unsigned int suppress)
 		     PL_FUNCTOR_CHARS, "discontiguous", 1,
 		       _PL_PREDICATE_INDICATOR, proc);
     } else if ( !hasProcedureSourceFile(sf, proc) )
-    { abolishProcedure(proc, def->module);
+    { if ( true(def, P_THREAD_LOCAL) )
+	return PL_error(NULL, 0, NULL, ERR_MODIFY_THREAD_LOCAL_PROC, proc);
+
+      abolishProcedure(proc, def->module);
 
       if ( def->references )
       { printMessage(ATOM_informational,
@@ -2669,6 +2673,8 @@ redefineProcedure(Procedure proc, SourceFile sf, unsigned int suppress)
       }
     }
   }
+
+  return TRUE;
 }
 
 
@@ -2792,6 +2798,27 @@ PRED_IMPL("$time_source_file", 3, time_source_file, PL_FA_NONDETERMINISTIC)
 }
 
 
+static bool
+clearInitialization(SourceFile sf)
+{ GET_LD
+  int rc = FALSE;
+
+  fid_t fid = PL_open_foreign_frame();
+  term_t name = PL_new_term_ref();
+  static predicate_t pred = NULL;
+
+  if ( !pred )
+    pred = PL_predicate("$clear_initialization", 1, "system");
+
+  PL_put_atom(name, sf->name);
+  rc = PL_call_predicate(MODULE_system, PL_Q_NORMAL, pred, name);
+
+  PL_discard_foreign_frame(fid);
+
+  return rc;
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 startConsult(SourceFile sf)
 
@@ -2816,6 +2843,8 @@ startConsult(SourceFile f)
     sigset_t set;
     ClauseRef garbage = NULL;
 
+    clearInitialization(f);
+
     LOCK();
     PL_LOCK(L_THREAD);
     blockSignals(&set);
